% s_CLARITYTractography.m
%
% This script reconstructs axons from 3D CLARITY data by: 
%
%   (1) Computing structure tensors 
%
%   (2) Performing tractography using Diffusion Toolkit and Trackvis
%   softwares
%
% Download and install required toolbox and softwares below:  
%   (1) Tools for NIfTI and ANALYZE image
%       http://www.mathworks.com/matlabcentral/fileexchange/8797-tools-for-nifti-and-analyze-image
%
%   (2) FMRIB Software Library
%       http://fsl.fmrib.ox.ac.uk/fsl/fslwiki/FslInstallation
%
%   (3) Diffusion Toolkit and Trackvis softwares
%       http://trackvis.org/
%   
%
% (c) Qiyuan Tian, McNab Lab, Stanford University, September 2015

clear, clc, close all

dpRoot = fileparts(which('s_CLARITYTractography.m')); % root directory path
cd(dpRoot);

addpath(genpath('NIfTI_20140122')); % add nifti toolbox to matlab path

%% Input parameters
% data info
% store data in daData, each within a folder named 'v??', e.g. v100. each
% data folder contains subfolders named ch0, ch1, brainmask, seedmask etc.
% each subfolder contains tiff images.
dpData = '/data/'; % data directory path
fnDataArr = dir(fullfile(dpData, 'v*')); % data filenames
chArr = {'ch0', 'ch1'};% ch0 and ch1 are green and red channel, respectively

% parameters
% the scrip loops through all paramerters and saves out each result
% automatically.
dogsigmaArr = [1]; % sigma values for derivitive of gaussian filter, recommended value: 0.6-1.3 (based on actual data)
gausigmaArr = [2.3]; % sigma values for gaussian filer, recommended value: 1.3-2.3 (based on actual data)
angleArr = [25]; % angle threhholds for fiber tracking. recommended value: 20-30

%% Compute structure tensors
disp('*****Start Compute Structure Tensors*****')

for ii = 1 : length(fnDataArr)
    fnData = fnDataArr(ii).name;
    disp(['*****Start ' fnData '*****']);
    
    % set up result directory
    dpResult = fullfile(dpData, fnData, 'result');
    if exist(dpResult, 'dir') % remove existing directory 
        rmdir(dpResult, 's');
    end
    mkdir(dpResult);
    
    for nn = 1 : length(chArr)
        ch = chArr{nn};
        dpDataCh = fullfile(dpData, fnData, ch); % directory path for data channel
        imgstack = single(imgfileread(dpDataCh)); % read image stack    

        for jj = 1 : length(dogsigmaArr)
            dogsigma = dogsigmaArr(jj);
            disp(['*****Start DoG Sigma ' num2str(dogsigma) '*****']);

            % generate dog kernels
            dogkercc = single(doggen([dogsigma, dogsigma, dogsigma]));
            dogkerrr = permute(dogkercc, [2, 1, 3]);
            dogkerzz = permute(dogkercc, [1, 3, 2]);

            % compute gradients
            grr = convn(imgstack, dogkerrr, 'same');
            gcc = convn(imgstack, dogkercc, 'same');
            gzz = convn(imgstack, dogkerzz, 'same');

            % compute gradient products
            gprrrr = grr .* grr;
            gprrcc = grr .* gcc;
            gprrzz = grr .* gzz;
            gpcccc = gcc .* gcc;
            gpcczz = gcc .* gzz;
            gpzzzz = gzz .* gzz;

            % compute gradient amplitudes
            ga = sqrt(gprrrr + gpcccc + gpzzzz);
            save_nii(make_nii(ga), fullfile(dpResult, [fnData '_' ch '_ga_dogsig' num2str(dogsigma) '.nii']));

            % compute gradient vectors
            gv = cat(4, grr, gcc, gzz);
            gv = gv ./ repmat(ga, [1, 1, 1, 3]);
            save_nii(make_nii(gv), fullfile(dpResult, [fnData '_' ch '_gv_dogsig' num2str(dogsigma) '.nii']));

            % compute structure tensors
            for kk = 1 : length(gausigmaArr)
                gausigma = gausigmaArr(kk);
                disp(['*****Start Gauss Sigma ' num2str(gausigma) '*****']);

                dpResultGau = fullfile(dpResult, ['dogsig' num2str(dogsigma) '_gausig' num2str(gausigma)]);
                mkdir(dpResultGau);
                
                % generate gaussian kernel
                gaussker = single(gaussgen([gausigma, gausigma, gausigma]));
                
                % blur gradient products
                gprrrrgauss = convn(gprrrr, gaussker, 'same');
                gprrccgauss = convn(gprrcc, gaussker, 'same');
                gprrzzgauss = convn(gprrzz, gaussker, 'same');
                gpccccgauss = convn(gpcccc, gaussker, 'same');
                gpcczzgauss = convn(gpcczz, gaussker, 'same');
                gpzzzzgauss = convn(gpzzzz, gaussker, 'same');
                
                % tensors in FSL format
                tensorfsl = cat(4, gprrrrgauss, gprrccgauss, gprrzzgauss, gpccccgauss, gpcczzgauss, gpzzzzgauss);
                fnTensorfsl = [fnData '_' ch '_tensorfsl_dogsig' num2str(dogsigma) '_gausig' num2str(gausigma) '.nii'];
                save_nii(make_nii(tensorfsl), fullfile(dpResultGau, fnTensorfsl));
                
                % tensors in DTK format
                tensordtk = cat(4, tensorfsl(:, :, :, 1:2), tensorfsl(:, :, :, 4), tensorfsl(:, :, :, 3), tensorfsl(:, :, :, 5:6));
                fnTensordtk = [fnData '_' ch '_tensordtk_dogsig' num2str(dogsigma) '_gausig' num2str(gausigma) '_tensor.nii'];
                save_nii(make_nii(tensordtk), fullfile(dpResultGau, fnTensordtk));
            end % gausigma        
        end % dogsigma
    end % channel
end % data

%% Convert tiff to nii
disp('*****Start Data Conversion*****')

for ii = 1 : length(fnDataArr)
    fnData = fnDataArr(ii).name;
    disp(['*****Start ' fnData '*****']);
    
    dpSubArr = dir(fullfile(dpData, fnData)); % sub directories

    for jj = 3 : length(dpSubArr) 
        fnDataSub = dpSubArr(jj).name;
        dpDataSub = fullfile(dp, datafn, fnDataSub);
        if isempty(dir(fullfile(dpDataSub, '*.nii')))
            imagestack = single(imgfileread(dpDataSub)); % read image stack
            save_nii(make_nii(imagestack), fullfile(dpDataSub, [fnData '_' fnDataSub '.nii'])); % save nii image
            clear imagestack
        end
    end
end

%% Perform tractography
disp('*****Start Tracking*****')

for ii = 1 : length(fnDataArr)
    fnData = fnDataArr(ii).name;
    disp(['*****Start ' fnData '*****']);
    
    fpMaskBrain = fullfile(dpData, fnData, 'brainmask', [fnData '_brainmask.nii']); % brain mask
    fpMaskSeed = fullfile(dpData, fnData, 'seedmask', [fnData '_seedmask.nii']); % seed mask
    dpResult = fullfile(dpData, fnData, 'result');
    
    for nn = 1 : length(chArr)
        ch = chArr{nn};
        
        for jj = 1 : length(dogsigmaArr)
            dogsigma = dogsigmaArr(jj);
            disp(['*****Start DoG Sigma ' num2str(dogsigma) '*****']);

            for kk = 1 : length(gausigmaArr)
                gausigma = gausigmaArr(kk);
                disp(['*****Start Gauss Sigma ' num2str(gausigma) '*****']);

                dpResultGau = fullfile(dpResult, ['dogsig' num2str(dogsigma) '_gausig' num2str(gausigma)]);
                fnTensordtk = [fnData  '_' ch '_tensordtk_dogsig' num2str(dogsigma) '_gausig' num2str(gausigma)];
                
                cd(dpResultGau)
                for aa = 1 : length(angleArr)
                    angle = angleArr(aa);
                    fnTrack = [fnTensordtk, '_track_seed_angle' num2str(angle) '.trk']; 
                    
                    % tracking 
                    cmd = ['dti_tracker ' fnTensordtk ' ' fnTrack ' -at ' num2str(angle) ' -v3 -m ' fpMaskBrain ' 0.1 1.1 -sm ' fpMaskSeed ' 0.1 1.1'];
                    [status, result] = system(cmd);
                end
                cd(dpRoot)
            end % gausigma        
        end % dogsigma
    end % channel 
end % data


